@@ -102,7 +102,8 @@ class TaskManager:
 
 _implemented_methods = (
     'set', 'get', 'clear',
-    'get_work_que', 'get_res_que', 'run',
+    'get_work_que', 'get_res_que',
+    'run', 'get_rpc_proxy'
 )
 
 
@@ -119,9 +120,9 @@ for _m in _implemented_methods:
 
 
 def rpc_client(server_id='default'):
-    clt = RPCClient(*rpc_connect_args(server_id=server_id))
-    clt.connect()
-    return clt
+    client = RPCClient(*rpc_connect_args(server_id=server_id))
+    client.connect()
+    return client
 
 
 def start_rpc_worker(methods_map: dict = None, *, topic='default', server_id='default', overwrite=True, **kwargs):
@@ -150,16 +151,47 @@ def start_rpc_worker(methods_map: dict = None, *, topic='default', server_id='de
                 que_ou.put(key, {'message': 'error', 'result': None})
 
 
+class MethodsProxy:
+    def __init__(self, cache_dic, task_mng):
+        self.cache_dict = cache_dic
+        self.task_manager = task_mng
+
+    def set(self, k, v):
+        self.cache_dict.__setitem__(k, v)
+
+    def get(self, k):
+        return self.cache_dict.get(k)
+
+    def clear(self):
+        self.cache_dict.clear()
+
+    def run(self, *args, **kwargs):
+        return self.task_manager.run(*args, **kwargs)
+
+    # def get_work_que(self, topic='default', methods=None, overwrite=True):
+    #     return self.task_manager.get_work_que(topic=topic, methods=methods, overwrite=overwrite)
+    #
+    # def get_res_que(self):
+    #     return self.task_manager.get_res_que()
+
+
 def start_rpc_server(retry_time=3, time_limit=30, server_id='default'):
     _task_manager = TaskManager(retry_time=retry_time, time_limit=time_limit)
     _cache_dict = {}
 
-    RPCServer.register('set', _cache_dict.__setitem__)
-    RPCServer.register('get', _cache_dict.get)
-    RPCServer.register('clear', _cache_dict.clear)
+    # RPCServer.register('set', _cache_dict.__setitem__)
+    # RPCServer.register('get', _cache_dict.get)
+    # RPCServer.register('clear', _cache_dict.clear)
     RPCServer.register('get_work_que', _task_manager.get_work_que)
     RPCServer.register('get_res_que', _task_manager.get_res_que)
-    RPCServer.register('run', _task_manager.run)
+    # RPCServer.register('run', _task_manager.run)
+
+    proxy = MethodsProxy(_cache_dict, _task_manager)
+    # for mtd in ('set', 'get', 'clear', 'get_work_que', 'get_res_que', 'run',):
+    for mtd in ('set', 'get', 'clear', 'run',):
+        RPCServer.register(mtd, proxy.__getattribute__(mtd))
+
+    RPCServer.register('get_rpc_proxy', lambda: proxy)
 
     print('RPC Server starting...')
     RPCServer(*rpc_connect_args(server=True, server_id=server_id)).get_server().serve_forever()
@@ -167,38 +199,62 @@ def start_rpc_server(retry_time=3, time_limit=30, server_id='default'):
 
 class RPCClt:
     def __init__(self, server_id='default'):
-        self.clt = RPCClient(*rpc_connect_args(server_id=server_id))
+        self.server_id = server_id
+        self.clt_raw = None
+        self.clt = None
         self.is_connected = False
 
+    def set_server_id(self, server_id):
+        self.server_id = server_id
+        return self
+
+    def __setitem__(self, key, value):
+        self.set(key, value)
+
+    def __getitem__(self, key):
+        return self.get(key)
+
     def check_connect(self):
         if not self.is_connected:
-            self.clt.connect()
+            self.clt_raw = RPCClient(*rpc_connect_args(server_id=self.server_id))
+            self.clt_raw.connect()
+            self.clt = self.clt_raw.get_rpc_proxy()
             self.is_connected = True
         return self
 
     def set(self, k, v):
         self.check_connect()
-        return self.clt.set(k, v)
+        self.clt.set(k, v)
 
     def get(self, k):
         self.check_connect()
+        # return self.clt.get(k)._getvalue()
         return self.clt.get(k)
 
     def clear(self):
         self.check_connect()
-        return self.clt.clear()
+        self.clt.clear()
 
     def run(self, data, *args, prior=None, timeout=None, topic='default', **kwargs):
         self.check_connect()
-        return self.clt.run(data, *args, prior=prior, timeout=timeout, topic=topic, **kwargs)
+        try:
+            # res = self.clt.run(data, *args, prior=prior, timeout=timeout, topic=topic, **kwargs)._getvalue()
+            res = self.clt.run(data, *args, prior=prior, timeout=timeout, topic=topic, **kwargs)
+            return res['result']
+        except Exception:
+            print(traceback.format_exc())
 
     def get_work_que(self, topic='default', methods=None, overwrite=True):
         self.check_connect()
-        return self.clt.get_work_que(topic=topic, methods=methods, overwrite=overwrite)
+        return self.clt_raw.get_work_que(topic=topic, methods=methods, overwrite=overwrite)
 
     def get_res_que(self):
         self.check_connect()
-        return self.clt.get_res_que()
+        return self.clt_raw.get_res_que()
+
+    def get_rpc_proxy(self):
+        self.check_connect()
+        return self.clt
 
 
 clt = RPCClt()
