@@ -27,7 +27,7 @@ def main():
     cmd = params.command
     if cmd == 'config':
         from adk.rpc.rpc_cfg import set_rpc_config
-        set_rpc_config(host=params.host, port=params.host, key=params.key,
+        set_rpc_config(host=params.host, port=params.port, key=params.key,
                        server_id=params.sid, filename=params.f)
     elif cmd == 'start':
         from adk.rpc.rpc_manager import start_rpc_server
