@@ -5,19 +5,21 @@ Main usage:
 
 TaskQue(
     maxsize=0,              max size of queue
+
     lock=None,              mutex lock for queue, Lock or RLock
     spd_lim=False,          use speed limit or not, i.e. for flow control
     spd_freq=0.1,           check speed every 'spd_freq' seconds
     max_puts=0,             max number of 'put' calls in 'spd_freq' seconds
     max_gets=0,             max number of 'get' calls in 'spd_freq' seconds
-    auto_key=False,         auto generate unique key for 'put' or not
+
+    auto_key=True,         auto generate unique key for 'put' or not
     use_retain=True,        retain data for further use (wait accept) after 'get' calls or not
     use_trace=True,         use trace (record timestamp and last_op...) or not
     default_cb=None,        default callback, i.e. {'timeout_cb':print}
     max_retain=10000,       max size of retain data
     retry_time=0,           if 'accept' isn't called in 'retry_time' seconds after 'get',
                             put the data back to queue
-    time_limit=None,        every data expire after 'time_limit' if 'use_coroutine'
+    time_limit=None,        every data expire after 'time_limit' seconds
 
     executor=None,          executor, run callbacks, default is new instance of ThreadCron
     workers=1,              number of threads, argument in ThreadCron
@@ -74,6 +76,7 @@ Change logs:
                         new features...
     2020-04-15:         new versions of PriorityQueue and PriorityDict
     2020-10-23:         drop something
+    2020-12-09:         in TaskQue, put(*args, prior=prior) will get *args
 ------------------------------------------------------------------------------------------------------------
 """
 
@@ -83,12 +86,13 @@ from collections import deque
 from contextlib import contextmanager
 from time import monotonic as _time, sleep as _sleep
 import threading
-from .priority_datatypes import PriorityQueue, PriorityDict
+from .priority_datatypes import PriorityQueue, PriorityDict, DEFAULT_MARKER
 from .cron_executor import ThreadCron
 
 __all__ = ['TaskQue', 'QueBase', 'PriQue', 'PriDict']
 
-_DEFAULT_MARKER = object()
+
+# DEFAULT_MARKER = object()
 
 
 class QueBase:
@@ -250,7 +254,7 @@ class QueBase:
     def put_nowait(self, *args, **kwargs):
         return self.put(*args, block=False, **kwargs)
 
-    def get(self, key=_DEFAULT_MARKER, block=True, timeout=None, wait_free=None, **kwargs):
+    def get(self, key=DEFAULT_MARKER, block=True, timeout=None, wait_free=None, **kwargs):
         with self.not_empty:
             _s_time = _time()
             if wait_free is None:
@@ -313,17 +317,17 @@ class QueBase:
                 self.not_full.notify()
             return res
 
-    def get_nowait(self, key=_DEFAULT_MARKER, **kwargs):
+    def get_nowait(self, key=DEFAULT_MARKER, **kwargs):
         return self.get(key=key, block=False, **kwargs)
 
-    def peek(self, key=_DEFAULT_MARKER, block=True, timeout=None):
+    def peek(self, key=DEFAULT_MARKER, block=True, timeout=None):
         with self.not_empty:
             _flag = self.not_empty.wait_for(self._get_enabled, timeout) if block else self._qsize()
             if not _flag:
                 raise TimeoutError if self._frozen else self.Empty
             return self._peek(key=key)
 
-    def peek_nowait(self, key=_DEFAULT_MARKER):
+    def peek_nowait(self, key=DEFAULT_MARKER):
         return self.peek(key=key, block=False)
 
     def _empty(self):
@@ -369,11 +373,11 @@ class QueBase:
         self.queue.append(*args)
 
     # Get an item from the queue
-    def _get(self, key=_DEFAULT_MARKER, **kwargs):
+    def _get(self, key=DEFAULT_MARKER, **kwargs):
         return self.queue.popleft()
 
     # Peek an item from the queue
-    def _peek(self, key=_DEFAULT_MARKER):
+    def _peek(self, key=DEFAULT_MARKER):
         pass
 
     # do something when received data from 'get'
@@ -386,7 +390,7 @@ class QueBase:
 
 
 class TaskQue(QueBase):
-    def _init(self, maxsize=0, auto_key=False, use_retain=True, use_trace=True, default_cb=None,
+    def _init(self, maxsize=0, auto_key=True, use_retain=True, use_trace=True, default_cb=None,
               max_retain=10000, retry_time=0, time_limit=None,
               # use_coroutine=True, loop=None, loop_freq=0.005,
               # cb_container='auto', cb_workers=1, cb_queue=None,
@@ -405,8 +409,12 @@ class TaskQue(QueBase):
         self.retry_time = retry_time
 
         self.data = {}
-        self.retain = PriorityDict(complex_prior=False, )
-        self.queue = PriorityDict(return_prior=True, complex_prior=False, )
+        # self.retain = PriorityDict(complex_prior=False, )
+        # self.queue = PriorityDict(return_prior=True, complex_prior=False, )
+        # self.retain = PriorityDict(format_in=True, format_ou=True)
+        # self.queue = PriorityDict(format_in=True, format_ou=True)
+        self.retain = PriorityDict()
+        self.queue = PriorityDict()
 
         self.trace = {}
         self.running_cbs = {}
@@ -452,16 +460,20 @@ class TaskQue(QueBase):
             if not 1 <= len(args) <= 2:
                 raise TypeError('put expected 1~2 arguments if auto_key, got %d' % len(args))
             key, value = self.uid_maker.__next__(), args[0]
-            prior = kwargs['prior'] if 'prior' in kwargs else \
-                args[-1] if len(args) == 2 else None
+            # prior = kwargs['prior'] if 'prior' in kwargs else \
+            #     args[-1] if len(args) == 2 else None
+            kvp = (key, *args)
         else:
             if not 2 <= len(args) <= 3:
                 raise TypeError('put expected 2~3 arguments if auto_key, got %d' % len(args))
             key, value = args[0], args[1]
-            prior = kwargs['prior'] if 'prior' in kwargs else \
-                args[-1] if len(args) == 3 else None
+            # prior = kwargs['prior'] if 'prior' in kwargs else \
+            #     args[-1] if len(args) == 3 else None
+            kvp = args
         res = {'uid': key,
-               'kvp': (key, value, prior),
+               # 'kvp': (key, value, prior),
+               'kvp': kvp,
+               'kw': {'prior': kwargs['prior']} if 'prior' in kwargs else {},
                'put_cb': kwargs.get('put_cb', None),
                'get_cb': kwargs.get('get_cb', None),
                'accept_cb': kwargs.get('accept_cb', None),
@@ -537,7 +549,7 @@ class TaskQue(QueBase):
         _typ = 'update' if key in self.data else 'put'
 
         self.data[key] = dic
-        self.queue.push(*dic['kvp'])
+        self.queue.push(*dic['kvp'], **dic['kw'])
 
         if self.use_trace:
             self._gen_trace(key)
@@ -565,12 +577,12 @@ class TaskQue(QueBase):
         self.run_callback(dic['put_cb'], dic['kvp'], _typ)
         return key
 
-    def _get(self, key=_DEFAULT_MARKER, typ=None):
+    def _get(self, key=DEFAULT_MARKER, typ=None):
         kvp = self.queue.pop(key=key)
         dic = self.data[kvp[0]]
 
-        _key = key or kvp[0]
-        _typ = typ or ('get' if key is _DEFAULT_MARKER else 'drop')
+        _key = key if key != DEFAULT_MARKER else kvp[0]
+        _typ = typ or ('get' if key == DEFAULT_MARKER else 'drop')
 
         if self.use_trace:
             self.trace[_key][_typ] = datetime.now().strftime('%H:%M:%S.%f')
@@ -580,8 +592,8 @@ class TaskQue(QueBase):
             if len(self.retain) >= self.max_retain:
                 _drop_kvp = self.retain.pop()
                 self._finalize(_drop_kvp[0], msg='clean')
-            # self.retain[_key] = dic
-            self.retain[_key] = (dic, None)
+            self.retain[_key] = dic
+            # self.retain[_key] = (dic, None)
 
             if self.retry_time:
                 # if self.loop_freq:
@@ -606,7 +618,7 @@ class TaskQue(QueBase):
         if not self.use_retain or _typ == 'drop':
             self._finalize(_key, cln=False, msg='ok' if _typ == 'get' else 'drop')
 
-        return kvp
+        return dic['kvp']
 
     def _accept(self, key):
         dic = self.data[key]  # may raise KeyError
@@ -638,7 +650,7 @@ class TaskQue(QueBase):
             if key in self.data and key in self.retain:
                 dic = self.data[key]
                 self.retain.pop(key)
-                self.queue.push(*dic['kvp'])
+                self.queue.push(*dic['kvp'], **dic['kw'])
                 if self.use_trace:
                     self.trace[key]['retry'] = datetime.now().strftime('%H:%M:%S.%f')
                     self.trace[key]['last_op'] = 'retry'
@@ -686,10 +698,10 @@ class PriQue(QueBase):
     def _put(self, *args):
         self.queue.push(*args)
 
-    def _get(self, key=_DEFAULT_MARKER, **kwargs):
+    def _get(self, key=DEFAULT_MARKER, **kwargs):
         return self.queue.popitem()
 
-    def _peek(self, key=_DEFAULT_MARKER):
+    def _peek(self, key=DEFAULT_MARKER):
         return self.queue.peek()
 
 
@@ -706,13 +718,12 @@ class PriDict(QueBase):
     def _put(self, *args):
         self.queue.push(*args)
 
-    def _get(self, key=_DEFAULT_MARKER, **kwargs):
+    def _get(self, key=DEFAULT_MARKER, **kwargs):
         return self.queue.pop(key=key)
 
-    def _peek(self, key=_DEFAULT_MARKER):
+    def _peek(self, key=DEFAULT_MARKER):
         return self.queue.peek(key=key)
 
-
 # def _cb_worker(que):
 #     while True:
 #         try:
